
`resetall

module Gowin_EMPU_M3_template (
  //GPIO
  GPIO,
  //Debug Access Port
  JTAG_3,   //trstn
  JTAG_4,   //NC
  JTAG_5,   //tdi
  JTAG_6,   //NC
  JTAG_7,   //tms   |       swdio
  JTAG_8,   //extFlash_sspi2_sio3/holdn
  JTAG_9,   //tck   |       swclk
  JTAG_10,  //extFlash_spi2_mosi/sio1
  JTAG_11,  //NC    |       
  JTAG_12,  //extFlash_spi2_sclk
  JTAG_13,  //tdo   |       swo
  JTAG_14,  //extFlash_spi2_miso/sio0
  JTAG_15,  //srstn
  JTAG_16,  //extFlash_spi2_cs
  JTAG_17,  //NC
  JTAG_18,  //extFlash_sspi2_sio2/WPn
  //UART0
  UART0RXD,
  UART0TXD,
  //UART1
  UART1RXD,
  UART1TXD,
  //I2C Master
  SCL,
  SDA,
  //SPI Master
  MOSI,
  MISO,
  SCLK,
  NSS,
  //SPI-Flash
  FLASH_SPI_HOLDN,
  FLASH_SPI_CSN,
  FLASH_SPI_MISO,
  FLASH_SPI_MOSI,
  FLASH_SPI_WPN,
  FLASH_SPI_CLK,
  //System Clock
  HCLK,
  //System Reset
  hwRstn
);

inout JTAG_3;
inout JTAG_4;
inout JTAG_5;
inout JTAG_6;
inout JTAG_7;
inout JTAG_8;
inout JTAG_9;
inout JTAG_10;
inout JTAG_11;
inout JTAG_12;
inout JTAG_13;
inout JTAG_14;
inout JTAG_15;
inout JTAG_16;
inout JTAG_17;
inout JTAG_18;
inout [15:0] GPIO;
input UART0RXD ;
output UART0TXD ;
input HCLK ;
input hwRstn ;
inout FLASH_SPI_HOLDN;
inout FLASH_SPI_CSN;
inout FLASH_SPI_MISO;
inout FLASH_SPI_MOSI;
inout FLASH_SPI_WPN;
inout FLASH_SPI_CLK;
input UART1RXD;
output UART1TXD;
inout SCL;
inout SDA;
output MOSI;   // SPI output
input MISO;    // SPI input
output SCLK;   // SPI clock
output NSS;    // SPI chip select

//-------------------------APB2 Extension----------------------//
wire APB2PCLK ;
wire APB2PRESET ;
wire APB2PENABLE ;
wire [7:0] APB2PADDR ;
wire APB2PWRITE ;
wire [31:0] APB2PWDATA ;
wire [3:0] APB2PSTRB ;
wire [2:0] APB2PPROT ;
wire APB2PSEL ;
wire [31:0] APB2PRDATA ;
wire APB2PREADY ;
wire APB2PSLVERR ;
wire PSEL_VALID_ES;
wire PENABLE_VALID_ES;

Gowin_APB2_Multiple apb2_multiple_template (
 .pclk(APB2PCLK), 
 .presetn(APB2PRESET),
 .psel(PSEL_VALID_ES), 
 .penable(PENABLE_VALID_ES),
 .pwrite(APB2PWRITE), 
 .paddr({4'b0000,APB2PADDR[7:2]}),  
 .pwdata(APB2PWDATA), 
 .prdata(APB2PRDATA)
);

Gowin_APB2_Decoder 
    #(.ADDR_WIDTH(8))
 u_multiple_1(
     .paddr_i        ({24'h0,APB2PADDR[7:0]}),
	 .penable_i      (APB2PENABLE           ),
	 .pprot_i        (APB2PPROT[0]          ),
	 .secure_i       (1'b0                  ),
	 .pready_i       (1'b1                  ),
	 .psel_valid_o   (PSEL_VALID_ES         ),//decoded psel to slave
     .penable_valid_o(PENABLE_VALID_ES      ),//decoded penable to slave
     .pready_o       (APB2PREADY            ),
     .psel_i         (APB2PSEL              )
  );
//-------------------------APB2 Extension----------------------//

//-------------------------AHB2 Extension----------------------//
wire [31:0] AHB2HRDATA;      // Data from slave to master
wire        AHB2HREADYOUT;   // Slave ready signal
wire [1:0]  AHB2HRESP;       // Slave response signal  
wire [1:0]  AHB2HTRANS;      // Transfer type
wire [2:0]  AHB2HBURST;      // Burst type
wire [3:0]  AHB2HPROT;       // Transfer protection bits
wire [2:0]  AHB2HSIZE;       // Transfer size
wire        AHB2HWRITE;      // Transfer direction
wire        AHB2HMASTLOCK;   // Transfer is a locked transfer
wire        AHB2HREADYMUX;   // Ready mux signal
wire [3:0]  AH2BHMASTER;     // Transfer is a locked transfer
wire [31:0] AHB2HADDR;       // Transfer address
wire [31:0] AHB2HWDATA;      // Data from master to slave
wire        AHB2HSEL;        // Select
wire        AHB2HCLK;        // Bus clock signal
wire        AHB2HRESET;      // Bus reset signal

Gowin_AHB_Multiple u_ahb_multiple
(
	.AHB_HRDATA(AHB2HRDATA),
	.AHB_HREADY(AHB2HREADYOUT),//ready signal, slave to MCU master, 1'b1
	.AHB_HRESP(AHB2HRESP),//respone signal, slave to MCU master
	.AHB_HTRANS(AHB2HTRANS),
	.AHB_HBURST(AHB2HBURST),
	.AHB_HPROT(AHB2HPROT),
	.AHB_HSIZE(AHB2HSIZE),
	.AHB_HWRITE(AHB2HWRITE),
	.AHB_HMASTLOCK(AHB2HMASTLOCK),
	.AHB_HMASTER(AH2BHMASTER),
	.AHB_HADDR(AHB2HADDR[11:0]),
	.AHB_HWDATA(AHB2HWDATA),
	.AHB_HSEL(AHB2HSEL),
	.AHB_HCLK(AHB2HCLK),
	.AHB_HRESETn(AHB2HRESET)
);
//-------------------------AHB2 Extension----------------------//

//-------------------------Gowin_EMPU_M3_Top----------------------//
wire clkin = HCLK;	//Input 50MHz
wire MCU_SYS_CLK;	//MCU 20MHz
wire RTCSRCCLK;     //RTC 3.072MHz

Gowin_rPLL u_Gowin_rPLL(
  .clkout(MCU_SYS_CLK),     //MCU 20MHz
  .clkoutd(RTCSRCCLK),      //RTC 3.072MHz
  .clkin(clkin)             //input 50MHz
);

wire LOCKUP;
wire HALTED;

Gowin_EMPU_M3_Top u_Gowin_EMPU_M3_Top (
  .LOCKUP(LOCKUP),
  .HALTED(HALTED),
  .GPIO(GPIO),
  .JTAG_3(JTAG_3),
  .JTAG_4(JTAG_4),
  .JTAG_5(JTAG_5),
  .JTAG_6(JTAG_6),
  .JTAG_7(JTAG_7),
  .JTAG_8(JTAG_8),
  .JTAG_9(JTAG_9),
  .JTAG_10(JTAG_10),
  .JTAG_11(JTAG_11),
  .JTAG_12(JTAG_12),
  .JTAG_13(JTAG_13),
  .JTAG_14(JTAG_14),
  .JTAG_15(JTAG_15),
  .JTAG_16(JTAG_16),
  .JTAG_17(JTAG_17),
  .JTAG_18(JTAG_18),
  .UART0RXD(UART0RXD),
  .UART0TXD(UART0TXD),
  .UART1RXD(UART1RXD),
  .UART1TXD(UART1TXD),
  .TIMER0EXTIN(1'b0),
  .TIMER1EXTIN(1'b0),
  .RTCSRCCLK(RTCSRCCLK),
  .SCL(SCL),
  .SDA(SDA),
  .MOSI(MOSI),
  .MISO(MISO),
  .SCLK(SCLK),
  .NSS(NSS),
  .FLASH_SPI_HOLDN(FLASH_SPI_HOLDN),
  .FLASH_SPI_CSN(FLASH_SPI_CSN),
  .FLASH_SPI_MISO(FLASH_SPI_MISO),
  .FLASH_SPI_MOSI(FLASH_SPI_MOSI),
  .FLASH_SPI_WPN(FLASH_SPI_WPN),
  .FLASH_SPI_CLK(FLASH_SPI_CLK),
  .APB2PADDR(APB2PADDR),
  .APB2PENABLE(APB2PENABLE),
  .APB2PWRITE(APB2PWRITE),
  .APB2PSTRB(APB2PSTRB),
  .APB2PPROT(APB2PPROT),
  .APB2PWDATA(APB2PWDATA),
  .APB2PSEL(APB2PSEL),
  .APB2PRDATA(APB2PRDATA),
  .APB2PREADY(APB2PREADY),
  .APB2PSLVERR(APB2PSLVERR),
  .APB2PCLK(APB2PCLK),
  .APB2PRESET(APB2PRESET),
  .AHB2HRDATA(AHB2HRDATA),       // Data from slave to master
  .AHB2HREADYOUT(AHB2HREADYOUT), // Slave ready signal, from slave, 1'b1
  .AHB2HRESP(AHB2HRESP),         // Slave response signal  
  .AHB2HTRANS(AHB2HTRANS),       // Transfer type
  .AHB2HBURST(AHB2HBURST),       // Burst type
  .AHB2HPROT(AHB2HPROT),         // Transfer protection bits
  .AHB2HSIZE(AHB2HSIZE),         // Transfer size
  .AHB2HWRITE(AHB2HWRITE),       // Transfer direction
  .AHB2HREADYMUX(AHB2HREADYMUX), // Ready mux
  .AHB2HMASTLOCK(AHB2HMASTLOCK), // Transfer is a locked transfer
  .AHB2HMASTER(AH2BHMASTER),     // Transfer is a locked transfer
  .AHB2HADDR(AHB2HADDR),         // Transfer address
  .AHB2HWDATA(AHB2HWDATA),       // Data from master to slave   AHB2HSEL,
  .AHB2HSEL(AHB2HSEL),           // Select
  .AHB2HCLK(AHB2HCLK),           // Clock
  .AHB2HRESET(AHB2HRESET),       // Reset
  .HCLK(MCU_SYS_CLK),
  .hwRstn(hwRstn)
);
//-------------------------Gowin_EMPU_M3_Top----------------------//

endmodule